<?php
class tc_httpxfile extends tc_base{
	function test(){
		global $HTTP_REQUEST;
		$HTTP_REQUEST = new HTTPRequest(); // reset

		$request = $HTTP_REQUEST;

		$this->assertNull(HTTPXFile::GetInstance());

		$request->setMethod("post");

		$this->assertNull(HTTPXFile::GetInstance());

		$request->setHeader("Content-Disposition",sprintf('attachment; filename="%s"',rawurlencode("hlavička.jpg")));

		$this->assertNotNull($file = HTTPXFile::GetInstance());
		$this->assertEquals("hlavicka.jpg",$file->getFileName());
		$this->assertEquals("hlavička.jpg",$file->getFileName(array("sanitize" => false)));
		$this->assertEquals(false,$file->chunkedUpload());

		$request->setHeader("Content-Range","bytes 0-100/256");

		$this->assertEquals(true,$file->chunkedUpload());
		$this->assertEquals(true,$file->firstChunk());
		$this->assertEquals(false,$file->lastChunk());

		$request->setHeader("Content-Range","bytes 200-255/256");
		$this->assertEquals(true,$file->chunkedUpload());
		$this->assertEquals(false,$file->firstChunk());
		$this->assertEquals(true,$file->lastChunk());

		$request->setHeader("Content-Range","bytes 100-200/256");
		$this->assertEquals(true,$file->chunkedUpload());
		$this->assertEquals(false,$file->firstChunk());
		$this->assertEquals(false,$file->lastChunk());

		$request->setHeader("Content-Range","bytes 0-255/256");
		$this->assertEquals(false,$file->chunkedUpload());
		$this->assertEquals(true,$file->firstChunk());
		$this->assertEquals(true,$file->lastChunk());
	}

	function test_getToken(){
		$req1 = new HTTPRequest();
		$req1->setMethod("post");
		$req1->setRemoteAddr("10.20.30.40");
		$req1->setHeader("Content-Disposition",'attachment; filename="hlava.jpg"');
		$req1->setHeader("Content-Range","bytes 0-100/256");

		$req2 = new HTTPRequest();
		$req2->setMethod("post");
		$req2->setRemoteAddr("10.20.30.40");
		$req2->setHeader("Content-Disposition",'attachment; filename="hlava.jpg"');
		$req2->setHeader("Content-Range","bytes 200-255/256");

		$file1 = HTTPXFile::GetInstance(array("request" => $req1));
		$file2 = HTTPXFile::GetInstance(array("request" => $req2));

		$this->assertEquals(32,strlen($file1->getToken()));
		$this->assertTrue($file1->getToken()==$file2->getToken());

		$req2->setHeader("Content-Range","bytes 200-255/3000"); // different total size
		$this->assertTrue($file1->getToken()!=$file2->getToken());

		$req2->setHeader("Content-Range","bytes 200-255/256");
		$this->assertTrue($file1->getToken()==$file2->getToken());

		$req2->setHeader("Content-Disposition",'attachment; filename="hlava.png"'); // different filename
		$file2 = HTTPXFile::GetInstance(array("request" => $req2));
		$this->assertTrue($file1->getToken()!=$file2->getToken());

		$req2->setHeader("Content-Disposition",'attachment; filename="hlava.jpg"');
		$file2 = HTTPXFile::GetInstance(array("request" => $req2));
		$this->assertTrue($file1->getToken()==$file2->getToken());

		$req2->setRemoteAddr("11.22.33.44"); // different remote address
		$this->assertTrue($file1->getToken()!=$file2->getToken());
		$this->assertTrue($file1->getToken(array("consider_remote_addr" => false))==$file2->getToken(array("consider_remote_addr" => false)));

		$req2->setRemoteAddr("10.20.30.40");
		$this->assertTrue($file1->getToken()==$file2->getToken());
	}

	function test_legacy_way(){
		global $HTTP_REQUEST,$_FILES;
		$HTTP_REQUEST = new HTTPRequest(); // reset

		$_FILES = null;

		$HTTP_REQUEST->setMethod("post");
		$HTTP_REQUEST->_HTTPRequest_headers = array("X-File-Name" => "hlava.jpg");
		$this->assertEquals("hlava.jpg",$HTTP_REQUEST->getHeader("x-file-name"));

		$GLOBALS["HTTP_RAW_POST_DATA"] = Files::GetFileContent("hlava.jpg",$err,$err_str);
		$this->assertTrue(strlen($HTTP_REQUEST->getRawPostData())>0);

		$xfile = HTTPXFile::GetInstance(array("name" => "file.jpg"));
		$this->assertTrue(is_object($xfile));
		$this->assertFalse($xfile->chunkedUpload());
		$xfile->cleanUp();

		$file = $HTTP_REQUEST->getUploadedFile("file");
		$this->assertTrue(is_object($file));

		$this->assertTrue($file->isImage());
		$this->assertEquals(325,$file->getImageWidth());
		$this->assertEquals(448,$file->getImageHeight());

		$this->assertEquals("file",$file->getName());
		$this->assertEquals("hlava.jpg",$file->getFileName());

		$tmp_file = $file->getTmpFileName();
		$this->assertTrue(file_exists($tmp_file));
		$file->cleanUp();
		$this->assertFalse(file_exists($tmp_file));

		// cisteni
		$HTTP_REQUEST->setMethod("get");
		$HTTP_RAW_POST_DATA = null;
		$HTTP_REQUEST->_HTTPRequest_headers = array();
		$this->assertNull(HTTPXFile::GetInstance(array("name" => "file.jpg")));
	}
}
